Package: gargle
Title: Utilities for Working with Google APIs
Version: 1.5.2
Authors@R: c(
    person("Jennifer", "Bryan", , "jenny@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6983-2759")),
    person("Craig", "Citro", , "craigcitro@google.com", role = "aut"),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Google Inc", role = "cph"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Provides utilities for working with Google APIs
    <https://developers.google.com/apis-explorer>.  This includes
    functions and classes for handling common credential types and for
    preparing, executing, and processing HTTP requests.
License: MIT + file LICENSE
URL: https://gargle.r-lib.org, https://github.com/r-lib/gargle
BugReports: https://github.com/r-lib/gargle/issues
Depends: R (>= 3.6)
Imports: cli (>= 3.0.1), fs (>= 1.3.1), glue (>= 1.3.0), httr (>=
        1.4.5), jsonlite, lifecycle, openssl, rappdirs, rlang (>=
        1.1.0), stats, utils, withr
Suggests: aws.ec2metadata, aws.signature, covr, httpuv, knitr,
        rmarkdown, sodium, spelling, testthat (>= 3.1.7)
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
Author: Jennifer Bryan [aut, cre] (<https://orcid.org/0000-0002-6983-2759>),
  Craig Citro [aut],
  Hadley Wickham [aut] (<https://orcid.org/0000-0003-4757-117X>),
  Google Inc [cph],
  Posit Software, PBC [cph, fnd]
Maintainer: Jennifer Bryan <jenny@posit.co>
Built: R 4.4.0; ; 2024-04-30 17:58:15 UTC; unix
