# Generated by roxygen2: do not edit by hand

S3method(format,gargle_oauth_dat)
S3method(gargle_map_cli,"NULL")
S3method(gargle_map_cli,character)
S3method(gargle_map_cli,default)
S3method(print,gargle_oauth_client)
S3method(print,gargle_oauth_dat)
export(AuthState)
export(Gargle2.0)
export(GceToken)
export(WifToken)
export(bulletize)
export(check_is_service_account)
export(cred_funs_add)
export(cred_funs_clear)
export(cred_funs_list)
export(cred_funs_list_default)
export(cred_funs_set)
export(cred_funs_set_default)
export(credentials_app_default)
export(credentials_byo_oauth2)
export(credentials_external_account)
export(credentials_gce)
export(credentials_service_account)
export(credentials_user_oauth2)
export(field_mask)
export(gargle2.0_token)
export(gargle_api_key)
export(gargle_app)
export(gargle_client)
export(gargle_error_message)
export(gargle_map_cli)
export(gargle_oauth_cache)
export(gargle_oauth_client)
export(gargle_oauth_client_from_json)
export(gargle_oauth_client_type)
export(gargle_oauth_email)
export(gargle_oauth_sitrep)
export(gargle_oob_default)
export(gargle_verbosity)
export(gce_access_token)
export(gce_instance_service_accounts)
export(init_AuthState)
export(local_cred_funs)
export(local_gargle_verbosity)
export(oauth_app_from_json)
export(oauth_external_token)
export(request_build)
export(request_develop)
export(request_make)
export(request_retry)
export(response_as_json)
export(response_process)
export(secret_decrypt_json)
export(secret_encrypt_json)
export(secret_has_key)
export(secret_make_key)
export(secret_read_rds)
export(secret_write_rds)
export(tidyverse_api_key)
export(tidyverse_app)
export(tidyverse_client)
export(token_email)
export(token_fetch)
export(token_tokeninfo)
export(token_userinfo)
export(with_cred_funs)
export(with_gargle_verbosity)
import(fs)
import(rlang)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(glue,glue_data)
importFrom(lifecycle,deprecated)
